package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;
import java.util.Date;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.api.chart.SeriesType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class DetailedTimeSeriesChart extends AbstractBuilder {

	public DetailedTimeSeriesChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private DetailedTimeSeriesOptions options;
	private Dictionary<DetailedTimeSeriesChart> dictionary;
	
	public DetailedTimeSeriesSeries addSeries() {
		String id = addInvocation("addSeries");
		return new DetailedTimeSeriesSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public DetailedTimeSeriesOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new DetailedTimeSeriesOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<DetailedTimeSeriesChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<DetailedTimeSeriesChart>(this, id, "AuxLine");
	}
	
	public Dictionary<DetailedTimeSeriesChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.TIMESERIES_DATETIME;
		return r;
	}
	
	public static class DetailedTimeSeriesSeries extends AbstractSeries<DetailedTimeSeriesChart,DetailedTimeSeriesSeries> {

		public DetailedTimeSeriesSeries(DetailedTimeSeriesChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public DetailedTimeSeriesSeries setHideDataLabels(Boolean hideDataLabels) {
			
			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		
		
		public LimitToMeasure<DetailedTimeSeriesSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {
			
			String id = addInvocation("setLimitTo",limitPosition, amount, limitType);
			return new LimitToMeasure<DetailedTimeSeriesSeries>(this, id, "LimitToMeasure");
		}
		
		public DetailedTimeSeriesSeries setAxisX(String axisX) {

			addInvocation("setAxisX",axisX);
			return this;
		}
		
		public AggrFieldAxis<DetailedTimeSeriesSeries> setAxisY(String axisY) {
			
			String id = addInvocation("setAxisY",axisY);
			return new AggrFieldAxis<DetailedTimeSeriesSeries>(this, id, "AxisY");
		}

		public DetailedTimeSeriesSeries setType(SeriesType seriesType) {
			
			addInvocation("setType",seriesType);
			return this;
		}
		
		public DetailedTimeSeriesSeries setComparisonDim(String comparisonDimension) {
			
			addInvocation("setComparisonDim",comparisonDimension);
			return this;
		}
		
		public DetailedTimeSeriesAdditionalMeasure<DetailedTimeSeriesSeries> addAdditionalMeasure() {
			
			String id = addInvocation("addAdditionalMeasure");
			return new DetailedTimeSeriesAdditionalMeasure<DetailedTimeSeriesSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<DetailedTimeSeriesSeries> setFormula() {
			
			String id = addInvocation("setFormula");
			return new Formula<DetailedTimeSeriesSeries>(this, id, "Formula");
		}
		
		public DetailedTimeSeriesSeries setGeneratedQueryDto(String generatedQueryDto) {
			
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}

	}
	
	public static class DetailedTimeSeriesOptions extends AbstractOptions<DetailedTimeSeriesChart,DetailedTimeSeriesOptions> {

		public DetailedTimeSeriesOptions(DetailedTimeSeriesChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public DetailedTimeSeriesOptions setXMin(Date xMin) {
			
			addInvocation("setXMin",xMin);
			return this;
		}
		
		public DetailedTimeSeriesOptions setXMax(Date xMax) {
			
			addInvocation("setXMax",xMax);
			return this;
		}
	}
	
	public static class DetailedTimeSeriesAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,DetailedTimeSeriesAdditionalMeasure<ATTACHEDTO>> {

		public DetailedTimeSeriesAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}

		
		public Expression<DetailedTimeSeriesAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			String id = addInvocation("withExpression");
			return new Expression<DetailedTimeSeriesAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}
	}
}